//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalPnAlgo.h --- C++/F interface from CAL to PN gain/CTE correction codes
//
//	DESCRIPTION:
//		This header file acts as the interface between the SAS CAL and
//		the Fortran routines evgn01/evcr03 to perform gain and CTI
//		correction for EPIC PN. The package cfortran.h is used
//		for this purpose. The macros PROTOCCALSSFUBS... below declare
//		C++ callable wrapper functions to evgn01/evcr03 and four
//		added initialization routines which are needed to load
//		calibration data from the CCF into common blocks.
//		The macros CALPNALGO_... are to be used in the Cal.
//
//	CHANGE HISTORY:
//		UL, 1999-06-16: 1.0.0: - first version
//		UL, 1999-11-24: 2.0.0: - adapted to MPE v2 delivery:
//									+ three new paramaters to initialize gain
//									  correction
//									+ CCD mode and temperature to initialize
//									  CTE correction
//		UL, 2000-03-20: 2.1.0: - I/F change to evgn01:
//									+ the total charge is no longer needed as
//									  argument
//									+ the y-coordinate is now needed
//									+ routine returns now an approximate
//									  gain+CTI corrected amplitude
//		UL, 2000-07-31: 2.2.0: - I/F change in evgb01:
//									+ type input amplitude changed from INT
//									  to FLAT; name changed from IAMP to AMPR
//		UL, 2001-04-23: 2.3.0: - I/F change in getpnresp:
//									+ added cmode parameter
//							   - I/F change in CALPNALGO_initCteParam:
//									+ added ytiming parameter
//		UL, 2002-01-14: 2.4.0: - I/F change in CALPNALGO_getpnrsp
//									+ added pattern/pattfracchan/pattfraction
//									  parameter
//									
//	AUTHOR:
//		U. Lammers, Wed Jun 16 14:26:48 MET DST 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/calpnalgo/interface/CalPnAlgo.h,v 1.1 2000/08/31 13:49:48 ulammers Exp $
//

#include "common.h"
#include "cfortran.h"

#ifndef CALPNALGO_H
#define CALPNALGO_H


#if defined(F90MODPROC)
	#undef F90MODPROC
#endif

#define F90MODPROC(a, b)    CXXFUNC(b, b)

//
// -----------------------( Gain correction )-----------------------------
//
//
//	CALPNALGO_initGain:	initialize the gain correction; calling this
//						is a pre-requisite for CALPNALGO_gainCorrect
//
PROTOCCALLSFSUB9(F90MODPROC(calpnalgo, evgn01init), evgn01init,			\
				 INT, FLOATV, FLOATV, FLOATV, FLOATV, FLOATV, FLOATV,	\
				 INT, FLOATV)

#define CALPNALGO_initGain(N, PP0, PP1, PP2, PP3, AA, BB, M, GN)		\
    CCALLSFSUB9(EVGN01INIT, evgn01init, INT, FLOATV, FLOATV, FLOATV,	\
				FLOATV, FLOATV, FLOATV,	INT, FLOATV,					\
				N, PP0, PP1, PP2, PP3, AA, BB, M, GN)

//
//	CALPNALGO_gainCorrect: the gain correction; the gain correction
//						   routine; works on an event-by-event basis;
//						   for the meaning of the parameters, see
//						   comment section before `evgn01' in evgn01.f.
//
PROTOCCALLSFSUB8(F90MODPROC(calpnalgo, evgn01), evgn01,					\
				 INT, INT, INT, INT, FLOAT, PFLOAT, PFLOAT, PINT)

#define CALPNALGO_gainCorrect(IQUAD, ICCD, ICX, ICY, AMPR, AMPC, AMPCC, IERR)\
    CCALLSFSUB8(EVGN01, evgn01, INT, INT, INT, INT,	FLOAT, PFLOAT, PFLOAT, \
				PINT,\
				IQUAD, ICCD, ICX, ICY, AMPR, AMPC, AMPCC, IERR)


//
// -----------------------( CTE correction )-----------------------------
//
//
//	CALPNALGO_initCteParam: define CTE model parameters; calling this
//							is a pre-requisite for CALPNALGO_ctiCorrect
//
PROTOCCALLSFSUB8(F90MODPROC(calpnalgo, evcr03initcteparam),				\
							evcr03initcteparam,							\
                 INT, INTV, FLOATV, FLOATV, FLOATV, FLOAT, INT, FLOAT)

#define CALPNALGO_initCteParam(N, IFLPARIN, AIN, BIN, CIN, TEMP, MODE,	\
							   YTIMING)									\
    CCALLSFSUB8(EVCR03INITCTEPARAM, evcr03initcteparam,					\
                INT, INTV, FLOATV, FLOATV, FLOATV, FLOAT, INT, FLOAT,	\
                N, IFLPARIN, AIN, BIN, CIN, TEMP, MODE, YTIMING)

//
//	CALPNALGO_initHotPixels: define list of hot pixels; calling this
//							 is a pre-requisite to CALPNALGO_ctiCorrect
//
PROTOCCALLSFSUB7(F90MODPROC(calpnalgo, evcr03inithotpixels),			\
							evcr03inithotpixels,						\
				 INT, INTV, INTV, INTV, INTV, FLOATV, INTV)

#define CALPNALGO_initHotPixels(N, IQIN, ICIN, ICHIN, IYIN, ELIN, NHIN)	\
    CCALLSFSUB7(EVCR03INITHOTPIXELS, evcr03inithotpixels,				\
				INT, INTV, INTV, INTV, INTV, FLOATV, INTV,				\
				N, IQIN, ICIN, ICHIN, IYIN, ELIN, NHIN)


//
//	CALPNALGO_discardedPixelsInit: define list of discarded pixels; calling
//								   this is a pre-requisite to
//								   CALPNALGO_ctiCorrect
//
PROTOCCALLSFSUB5(F90MODPROC(calpnalgo, evcr03initdiscardedpixels),		\
							evcr03initdiscardedpixels,					\
				 INT, INTV, INTV, INTV, INTV)

#define CALPNALGO_initDiscardedPixels(N, IQIN, ICIN, ICHIN, IYIN)		\
    CCALLSFSUB5(EVCR03INITDISCARDEDPIXELS, evcr03initdiscardedpixels,	\
				INT, INTV, INTV, INTV, INTV,							\
				N, IQIN, ICIN, ICHIN, IYIN)


//
//	CALPNALGO_ctiCorrect: CTI correct an event; for the meaning of
//						  the arguments, see routine evcr03 in
//						  evcr03.f
//
PROTOCCALLSFSUB7(F90MODPROC(calpnalgo, evcr03), evcr03,					\
				 INT, INT, INT, INT, FLOAT, PFLOAT, PINT)

#define CALPNALGO_ctiCorrect(IFRM, IQUAD, IXQ, IYQ, ERAWP, ECOR, IERR)	\
    CCALLSFSUB7(EVCR03, evcr03,											\
				INT, INT, INT, INT, FLOAT, PFLOAT, PINT,				\
				IFRM, IQUAD, IXQ, IYQ, ERAWP, ECOR, IERR)

//
//	CALPNALGO_resp:       Get response; for the meaning of
//						  the arguments, see routine getpnrsp in
//						  getpnrsp.f
//
PROTOCCALLSFSUB17(F90MODPROC(calpnalgo, getpnrsp), getpnrsp,			\
				 FLOAT, FLOAT, INT, FLOATV, FLOATV, INT, STRING,		\
				 INT, INT, FLOATV,										\
				 INT, FLOATV, FLOATV, INT, FLOATV, FLOATV, FLOATV)

#define CALPNALGO_resp(ENERGY_LO, ENERGY_HI, NCHANNEL, EMIN, EMAX, LINE,\
				CMODE, PATTERN, PATTFRACCHAN, PATFRACTION,				\
				NSIABS, SIENERGY, SIABS, NSIO2ABS, SIO2ENERGY, SIO2ABS, \
				RESPONSE)												\
	CCALLSFSUB17(GETPNRSP, getpnrsp,									\
				FLOAT, FLOAT, INT, FLOATV, FLOATV, INT, STRING,			\
				INT, INT, FLOATV,										\
                INT, FLOATV, FLOATV, INT, FLOATV, FLOATV, FLOATV,		\
				ENERGY_LO, ENERGY_HI, NCHANNEL, EMIN, EMAX, LINE,		\
				CMODE, PATTERN, PATTFRACCHAN, PATFRACTION,				\
				NSIABS, SIENERGY, SIABS, NSIO2ABS, SIO2ENERGY, SIO2ABS, \
				RESPONSE)

#endif
